<?php

use Imoje\Payment\Util;
use JsonSchema\Validator;

/**
 * Class ControllerExtensionPaymentImoje
 */
class ControllerExtensionPaymentImoje extends Controller
{
	private $error = [];

	/**
	 * view admin panel
	 */
	public function index()
	{
		$this->language->load('extension/payment/imoje');
		$this->document->setTitle($this->language->get('heading_title'));
		$this->document->addScript('view/javascript/select2/js/select2.min.js');
		$this->document->addStyle('view/javascript/select2/css/select2.css');
		$this->load->model('setting/setting');

		$request = $this->request->post;

		if(($this->request->server['REQUEST_METHOD'] == 'POST')
			&& $this->validate($request)
		) {

			$settings = array_merge($this->model_setting_setting->getSetting('imoje'), $request);
			$this->model_setting_setting->editSetting('imoje', $settings);
			$this->session->data['success'] = $this->language->get('text_success');
			$this->response->redirect($this->url->link('extension/extension', 'token=' . $this->session->data['token'], 'SSL'));

			return;
		}

		include_once(DIR_SYSTEM . 'library/Payment-core/vendor/autoload.php');

		$data = [

			'button_save'   => $this->language->get('button_save'),
			'button_cancel' => $this->language->get('button_cancel'),

			'text_edit' => $this->language->get('text_edit'),

			'text_enabled'  => $this->language->get('text_enabled'),
			'text_disabled' => $this->language->get('text_disabled'),

			'text_header_tip' => $this->language->get('text_header_tip'),

			'imoje_status_label' => $this->language->get('imoje_status_label'),
			'imoje_status'       => isset($this->request->post['imoje_status'])
				? $this->request->post['imoje_status']
				: $this->config->get('imoje_status'),

			'imoje_debug_mode_label' => $this->language->get('imoje_debug_mode_label'),
			'imoje_debug_mode'       => isset($this->request->post['imoje_debug_mode'])
				? $this->request->post['imoje_debug_mode']
				: $this->config->get('imoje_debug_mode'),

			'imoje_sandbox_label'        => $this->language->get('imoje_sandbox_label'),
			'imoje_sandbox_enabled_hint' => $this->language->get('imoje_sandbox_enabled_hint'),
			'imoje_sandbox'              => isset($this->request->post['imoje_sandbox'])
				? $this->request->post['imoje_sandbox']
				: $this->config->get('imoje_sandbox'),

			'imoje_merchant_id_label' => $this->language->get('imoje_merchant_id_label'),
			'imoje_merchant_id'       => isset($this->request->post['imoje_merchant_id'])
				? $this->request->post['imoje_merchant_id']
				: $this->config->get('imoje_merchant_id'),

			'imoje_service_id_label' => $this->language->get('imoje_service_id_label'),
			'imoje_service_id'       => isset($this->request->post['imoje_service_id'])
				? $this->request->post['imoje_service_id']
				: $this->config->get('imoje_service_id'),

			'imoje_service_key_label' => $this->language->get('imoje_service_key_label'),
			'imoje_service_key'       => isset($this->request->post['imoje_service_key'])
				? $this->request->post['imoje_service_key']
				: $this->config->get('imoje_service_key'),

			'imoje_sort_order_label' => $this->language->get('imoje_sort_order_label'),
			'imoje_sort_order'       => isset($this->request->post['imoje_sort_order'])
				? $this->request->post['imoje_sort_order']
				: $this->config->get('imoje_sort_order'),

			'imoje_payment_title_default' => $this->language->get('imoje_payment_title_default'),
			'imoje_payment_title_label'   => $this->language->get('imoje_payment_title_label'),
			'imoje_payment_title'         => isset($this->request->post['imoje_payment_title'])
				? $this->request->post['imoje_payment_title']
				: $this->config->get('imoje_payment_title'),

			'imoje_currency_label' => $this->language->get('imoje_currency_label'),
			'imoje_all_currencies' => Util::getSupportedCurrencies(),
			'imoje_currencies'     => isset($this->request->post['imoje_currencies'])
				? $this->request->post['imoje_currencies']
				: ($this->config->get('imoje_currencies')
					?: []),

			'imoje_send_cart_label' => $this->language->get('imoje_send_cart_label'),
			'imoje_send_cart'       => isset($this->request->post['imoje_send_cart'])
				? $this->request->post['imoje_send_cart']
				: $this->config->get('imoje_send_cart'),

			'imoje_order_history_label' => $this->language->get('imoje_order_history_label'),
			'imoje_order_history'       => isset($this->request->post['imoje_order_history'])
				? $this->request->post['imoje_order_history']
				: $this->config->get('imoje_order_history'),

			'text_error_validation' => $this->error,

			'text_footer_notification' => $this->language->get('text_footer_notification'),
			'text_notification_url'    => HTTPS_CATALOG . 'index.php?route=extension/payment/imoje/notification',

			'confirm_url' => $this->url->link('extension/payment/imoje', 'token=' . $this->session->data['token'], true),
			'cancel_url'  => $this->url->link('extension/extension', 'token=' . $this->session->data['token'], true),

			'content_footer'      => $this->load->controller('common/footer'),
			'content_header'      => $this->load->controller('common/header'),
			'content_column_left' => $this->load->controller('common/column_left'),
			'heading_title'       => $this->language->get('text_heading_title'),

			'breadcrumbs' => [
				[
					'text'      => $this->language->get('text_home'),
					'href'      => $this->url->link('common/home', 'token=' . $this->session->data['token'], 'SSL'),
					'separator' => false,
				],

				[
					'text'      => $this->language->get('text_payment'),
					'href'      => $this->url->link('extension/extension', 'token=' . $this->session->data['token'], 'SSL'),
					'separator' => ' :: ',
				],

				[
					'text'      => $this->language->get('text_heading_title'),
					'href'      => $this->url->link('extension/payment/imoje', 'token=' . $this->session->data['token'], 'SSL'),
					'separator' => ' :: ',
				],
			],

		];

		$this->response->setOutput($this->load->view('extension/payment/imoje', $data));

		$current_imoje_settings = $this->model_setting_setting->getSetting('imoje');

		if(isset($current_imoje_settings['imoje_error_tmp'])) {
			unset($current_imoje_settings['imoje_error_tmp']);
			$this->model_setting_setting->editSetting('imoje', $current_imoje_settings);
		}

		if(isset($current_imoje_settings['imoje_success_tmp'])) {
			unset($current_imoje_settings['imoje_success_tmp']);
			$this->model_setting_setting->editSetting('imoje', $current_imoje_settings);
		}

		if(isset($current_imoje_settings['imoje_no_twisto'])) {
			unset($current_imoje_settings['imoje_no_twisto']);
			$this->model_setting_setting->editSetting('imoje', $current_imoje_settings);
		}
	}

	/**
	 * @param array $request
	 *
	 * @return bool
	 */
	protected function validate($request)
	{
		if(!$this->user->hasPermission('modify', 'extension/payment/imoje')) {
			$this->error['error_warning_permissions'] = $this->language->get('error_permission');

			return false;
		}

		include_once(DIR_SYSTEM . 'library/Payment-core/vendor/autoload.php');

		$schema = [
			'type'       => 'object',
			'properties' => [

				'imoje_status'        => [
					'type' => 'string',
					"enum" => [
						'0',
						'1',
					],
				],
				'imoje_debug_mode'    => [
					'type' => 'string',
					"enum" => [
						'0',
						'1',
					],
				],
				'imoje_sandbox'       => [
					'type' => 'string',
					"enum" => [
						'0',
						'1',
					],
				],
				'imoje_merchant_id'   => [
					'type' => 'string',
				],
				'imoje_service_id'    => [
					'type' => 'string',
				],
				'imoje_service_key'   => [
					'type' => 'string',
				],
				'imoje_sort_order'    => [
					'type'      => 'string',
					'minLength' => 1,
				],
				'imoje_payment_title' => [
					'type' => 'string',
				],
				'imoje_currencies'    => [
					'type'        => 'array',
					'items'       => [
						'type' => 'string',
						'enum' => array_values(\Imoje\Payment\Util::getSupportedCurrencies()),
					],
					'minItems'    => 1,
					'uniqueItems' => true,
				],
				'imoje_order_history' => [
					'type' => 'string',
					"enum" => [
						'0',
						'1',
					],
				],
				'imoje_send_cart'     => [
					'type' => 'string',
					"enum" => [
						'0',
						'1',
					],
				],
			],

			'required'             => [
				'imoje_status',
				'imoje_debug_mode',
				'imoje_sandbox',
				'imoje_merchant_id',
				'imoje_service_id',
				'imoje_service_key',
				'imoje_sort_order',
				'imoje_payment_title',
				'imoje_currencies',
				'imoje_send_cart',
				'imoje_order_history',
			],
			'additionalProperties' => false,
		];

		$data = json_decode(json_encode($request));

		$validator = new Validator();
		$validator->validate($data, $schema);

		$this->error = [];

		foreach($validator->getErrors() as $key => $value) {

			if($value['constraint'] === 'additionalProp') {
				$this->error['error_additional_properties'] = $this->language->get('error_additional_properties');
			} else {

				$this->error['error_' . $value['property']] = $this->language->get('error_validation');
			}
		}

		return empty($this->error);
	}
}
