<?php

use Imoje\Payment\CartData;
use Imoje\Payment\Notification;
use Imoje\Payment\Paywall;
use Imoje\Payment\Util;

/**
 * Class ControllerExtensionPaymentImoje
 */
class ControllerExtensionPaymentImoje extends Controller
{

	/**
	 * plugin version
	 */
	const VERSION = '2.0.1';

	/**
	 * @return mixed
	 */
	public function index()
	{

		$this->language->load('extension/payment/imoje');
		$this->load->model('setting/setting');

		$data = [
			'url_pay'        => $this->url->link('extension/payment/imoje/pay', '', true),
			'pay_with_imoje' => $this->language->get('pay_with_imoje'),
			'error_pay'      => $this->language->get('error_pay'),
		];

		return $this->load->view('extension/payment/imoje', $data);
	}

	/**
	 * Initiate form payment, make payment
	 */
	public function pay()
	{

		include_once(DIR_SYSTEM . 'library/Payment-core/vendor/autoload.php');

		$this->load->model('checkout/order');
		$this->language->load('extension/payment/imoje');
		$this->load->model('setting/setting');

		$order_info = $this->model_checkout_order->getOrder($this->session->data['order_id']);

		$paywall = new Paywall(
			($this->config->get('imoje_sandbox') == 1
				? Util::ENVIRONMENT_SANDBOX
				: Util::ENVIRONMENT_PRODUCTION),
			$this->config->get('imoje_service_key'),
			$this->config->get('imoje_service_id'),
			$this->config->get('imoje_merchant_id')
		);

		$cart_data = '';

		if($this->config->get('imoje_send_cart')) {

			$cart_data = $this->getCart($order_info);
		}

		$order_data = Paywall::prepareData(
			Util::convertAmountToFractional($order_info['total'] * $order_info['currency_value']),
			strtoupper($order_info['currency_code']),
			$order_info['order_id'],
			$order_info['payment_firstname'],
			$order_info['payment_lastname'],
			'',
			$order_info['email'],
			$order_info['telephone'],
			$this->url->link('checkout/success'),
			$this->url->link('checkout/success'),
			$this->url->link('checkout/success'),
			self::VERSION . ';opencart_' . VERSION,
			$cart_data
		);

		$data = [
			'form' => $paywall->buildOrderForm(
				$order_data,
				$this->language->get('submit')
			),

			'text_tip_checkout' => $this->language->get('text_tip_checkout'),
		];

		$this->model_checkout_order->addOrderHistory($order_info['order_id'], 01);

		$this->load->language('checkout/cart');
		$this->cart->clear();

		unset($this->session->data['vouchers']);
		unset($this->session->data['shipping_method']);
		unset($this->session->data['shipping_methods']);
		unset($this->session->data['payment_method']);
		unset($this->session->data['payment_methods']);
		unset($this->session->data['reward']);

		$this->response->setOutput($this->load->view('extension/payment/imoje_redirect', $data));
	}

	/**
	 * Support incoming notification (and validate)
	 *
	 * @throws Exception
	 */
	public function notification()
	{

		include_once(DIR_SYSTEM . 'library/Payment-core/vendor/autoload.php');

		$notification = new Notification(
			$this->config->get('imoje_service_id'),
			$this->config->get('imoje_service_key')
		);

		$result_check_request_notification = $notification->checkRequest();

		$debug_mode = $this->config->get('imoje_debug_mode');

		if(is_int($result_check_request_notification)) {
			echo Notification::formatResponse(Notification::NS_ERROR, $result_check_request_notification, $debug_mode);
			exit();
		}

		$this->load->model('checkout/order');

		$order = $this->model_checkout_order->getOrder($result_check_request_notification['transaction']['orderId']);

		if(!$order) {
			echo Notification::formatResponse(Notification::NS_ERROR, Notification::NC_ORDER_NOT_FOUND, $debug_mode);
			exit();
		}

		if(!Notification::checkRequestAmount(
			$result_check_request_notification,
			Util::convertAmountToFractional($order['total'] * $order['currency_value']),
			strtoupper($order['currency_code']))) {

			echo Notification::formatResponse(Notification::NS_ERROR, Notification::NC_AMOUNT_NOT_MATCH, $debug_mode);
			exit();
		}

		$orderStatusBefore = $order['order_status_id'];
		$orderStatusAfter = null;

		if($order['order_status_id'] === '1' || $order['order_status_id'] === '10') {

			switch($result_check_request_notification['transaction']['status']) {
				case 'settled':
					$orderStatusAfter = 02;
					$this->model_checkout_order->addOrderHistory($order['order_id'], $orderStatusAfter);
					break;
				case 'rejected':
					$orderStatusAfter = 10;
					$this->model_checkout_order->addOrderHistory($order['order_id'], $orderStatusAfter);
					break;
				default:
					echo Notification::formatResponse(Notification::NS_OK, Notification::NC_ORDER_STATUS_NOT_CHANGED, $debug_mode, $orderStatusBefore);
					exit();
			}
		}

		echo Notification::formatResponse(Notification::NS_OK, Notification::NC_OK, $debug_mode, $orderStatusBefore, $orderStatusAfter);
	}

	/**
	 * @param array $order_id
	 *
	 * @return string
	 */
	private function getOrderData($order_id)
	{

		include_once(DIR_SYSTEM . 'library/Payment-core/vendor/autoload.php');

		$this->load->model('catalog/product');
		$this->load->model('account/order');

		$cart_data = new CartData();

		$order = $this->model_account_order->getOrder($order_id);

		$total = 0;

		foreach($this->model_account_order->getOrderProducts($order_id) as $order_item) {

			$price = Util::convertAmountToFractional($this->tax->calculate(
					$order_item['price'],
					$order_item['tax_class_id'],
					$this->config->get('config_tax')
				) * $order['currency_value']);

			if($order_item['special']) {
				$price = Util::convertAmountToFractional($this->tax->calculate(
						$order_item['special'],
						$order_item['tax_class_id'],
						$this->config->get('config_tax')
					) * $order['currency_value']);
			}

			$quantity = (float) $order_item['quantity'];

			$cart_data->addItem(
				$order_item['product_id'],
				(float) current($this->tax->getRates($order_item['price'], $order_item['tax_class_id']))['rate'],
				$order_item['name'],
				$price,
				$quantity
			);

			$total += $price * $quantity;
		}

		$cart_data->setShipping(
			0,
			'noshipping',
			0
		);

		$addressDelivery = $this->getAddressDelivery($order);

		$cart_data->setAddressDelivery(
			$addressDelivery['shipping_city'],
			$addressDelivery['shipping_name'],
			$addressDelivery['telephone'],

			$addressDelivery['shipping_address'],
			$addressDelivery['shipping_iso_code_2'],
			$addressDelivery['shipping_postcode']
		);

		$cart_data->setAddressBilling(
			$order['payment_city'],
			$order['payment_firstname'] . ' ' . $order['payment_lastname'],
			$order['telephone'],
			isset($order['payment_address_2'])
				? $order['payment_address_1'] . ' ' . $order['payment_address_2']
				: $order['payment_address_1'],
			$order['payment_iso_code_2'],
			$order['payment_postcode']
		);

		$cart_data->setAmount($total);
		$cart_data->setCreatedAt(strtotime($order['date_added']));

		return $cart_data;
	}

	/**
	 * @param array $order
	 *
	 * @return array
	 */
	private function getAddressDelivery($order)
	{

		// check if shipping data is provided - 'shipping_city' is enough
		if($order['shipping_city']) {
			return [
				'shipping_city'       => $order['shipping_city'],
				'shipping_name'       => $order['shipping_firstname'] . ' ' . $order['shipping_lastname'],
				'telephone'           => $order['telephone'],
				'shipping_address'    => isset($order['shipping_address_2'])
					? $order['shipping_address_1'] . ' ' . $order['shipping_address_2']
					: $order['shipping_address_1'],
				'shipping_iso_code_2' => $order['shipping_iso_code_2'],
				'shipping_postcode'   => $order['shipping_postcode'],
			];
		} else {
			return [
				'shipping_city'       => $order['payment_city'],
				'shipping_name'       => $order['payment_firstname'] . ' ' . $order['payment_lastname'],
				'telephone'           => $order['telephone'],
				'shipping_address'    => isset($order['payment_address_2'])
					? $order['payment_address_1'] . ' ' . $order['payment_address_2']
					: $order['payment_address_1'],
				'shipping_iso_code_2' => $order['payment_iso_code_2'],
				'shipping_postcode'   => $order['payment_postcode'],
			];
		}
	}

	/**
	 * @param string $email
	 *
	 * @return array
	 */
	private function getPreviousOrders($email)
	{

		include_once(DIR_SYSTEM . 'library/Payment-core/vendor/autoload.php');

		$sql = "
SELECT `o`.`order_id`
FROM `oc_order` `o`
WHERE `o`.`email` = '" . $this->db->escape($email) . "'
  AND (`order_status_id` = 5 OR `order_status_id` = 15)
LIMIT " . CartData::MAX_PREVIOUS_ORDERS . ";
";

		$query = $this->db->query($sql);
		$rows = $query->rows;

		$orders = [];
		foreach($rows as $row) {
			$orders[] = $this->getOrderData($row['order_id']);
		}

		return $orders;
	}

	/**
	 * @param array $order
	 *
	 * @return string
	 */
	private function getCart($order)
	{

		include_once(DIR_SYSTEM . 'library/Payment-core/vendor/autoload.php');

		$this->load->model('catalog/product');

		$cart_data = new CartData();

		if($this->config->get('imoje_order_history')) {

			$previous_orders = $this->getPreviousOrders($order['email']);

			foreach($previous_orders as $previous_order) {
				$cart_data->addPrevious($previous_order);
			}
		}

		$cart_data->setAmount(Util::convertAmountToFractional($order['total'] * $order['currency_value']));
		$cart_data->setCreatedAt(strtotime($order['date_added']));

		foreach($this->cart->getProducts() as $cart_item) {

			$price = $this->tax->calculate(
				$cart_item['price'],
				$cart_item['tax_class_id'],
				$this->config->get('config_tax'));

			if($cart_item['special']) {
				$price = $this->tax->calculate(
					$cart_item['special'],
					$cart_item['tax_class_id'],
					$this->config->get('config_tax'));
			}

			$cart_data->addItem(
				$cart_item['product_id'],
				(float) current($this->tax->getRates($cart_item['price'], $cart_item['tax_class_id']))['rate'],
				$cart_item['name'],
				Util::convertAmountToFractional($price * $order['currency_value']
				),
				(float) $cart_item['quantity']
			);
		}

		$shipping_data = $this->session->data['shipping_method'];

		if($shipping_data) {
			$cart_data->setShipping(
				Util::convertAmountToFractional(
					current($this->tax->getRates($shipping_data['cost'], $shipping_data['tax_class_id']))['rate']
				),
				$shipping_data['title'],
				Util::convertAmountToFractional(
					$this->tax->calculate(
						$shipping_data['cost'],
						$shipping_data['tax_class_id'],
						$this->config->get('config_tax')) * $order['currency_value']
				)
			);
		}

		$cart_data->setAddressBilling(
			$order['payment_city'],
			$order['payment_firstname'] . ' ' . $order['payment_lastname'],
			$order['telephone'],
			isset($order['payment_address_2'])
				? $order['payment_address_1'] . ' ' . $order['payment_address_2']
				: $order['payment_address_1'],
			$order['payment_iso_code_2'],
			$order['payment_postcode']
		);

		$addressDelivery = $this->getAddressDelivery($order);

		$cart_data->setAddressDelivery(
			$addressDelivery['shipping_city'],
			$addressDelivery['shipping_name'],
			$addressDelivery['telephone'],

			$addressDelivery['shipping_address'],
			$addressDelivery['shipping_iso_code_2'],
			$addressDelivery['shipping_postcode']
		);

		$this->load->model('extension/total/coupon');

		if(isset($this->session->data['coupon'])) {
			$coupon_info = $this->model_extension_total_coupon->getCoupon($this->session->data['coupon']);

			if($coupon_info['total'] > 0) {
				$cart_data->setDiscount(
					0,
					'discount',
					Util::convertAmountToFractional(
						$coupon_info['total'] * $order['currency_value']
					)
				);
			}
		}

		return $cart_data->prepareCartData();
	}
}
